@@ -174,7 +174,8 @@ static pj_status_t get_local_interface(const pj_sockaddr *server,
     int addr_len;
     pj_status_t status;
 
-    status = pj_sock_socket(server->addr.sa_family, pj_SOCK_DGRAM(),
+    status = pj_sock_socket(server->addr.sa_family,
+                            pj_SOCK_DGRAM() | pj_SOCK_CLOEXEC(),
                             0, &sock);
     if (status != PJ_SUCCESS)
         return status;
@@ -213,8 +214,9 @@ PJ_DEF(pj_status_t) pj_stun_detect_nat_type(const pj_sockaddr_in *server,
 {
     pj_sockaddr srv;
 
-    if (server)
-        pj_sockaddr_cp(&srv, server);
+    PJ_ASSERT_RETURN(server, PJ_EINVAL);
+
+    pj_sockaddr_cp(&srv, server);
 
     return pj_stun_detect_nat_type2(&srv, stun_cfg, user_data, cb);
 }
@@ -273,7 +275,7 @@ PJ_DEF(pj_status_t) pj_stun_detect_nat_type2(const pj_sockaddr *server,
      * Initialize socket.
      */
     af = server->addr.sa_family;
-    status = pj_sock_socket(af, pj_SOCK_DGRAM(), 0, &sess->sock);
+    status = pj_sock_socket(af, pj_SOCK_DGRAM() | pj_SOCK_CLOEXEC(), 0, &sess->sock);
     if (status != PJ_SUCCESS)
         goto on_error;
 
@@ -366,7 +368,7 @@ static void sess_destroy(nat_detect_session *sess)
         pj_ioqueue_unregister(sess->key);
         sess->key = NULL;
         sess->sock = PJ_INVALID_SOCKET;
-    } else if (sess->sock && sess->sock != PJ_INVALID_SOCKET) {
+    } else if (sess->sock != PJ_INVALID_SOCKET) {
         pj_sock_close(sess->sock);
         sess->sock = PJ_INVALID_SOCKET;
     }
@@ -412,8 +414,8 @@ static void end_session(nat_detect_session *sess,
     delay.sec = 0;
     delay.msec = 0;
 
-    sess->timer.id = TIMER_DESTROY;
-    pj_timer_heap_schedule(sess->timer_heap, &sess->timer, &delay);
+    pj_timer_heap_schedule_w_grp_lock(sess->timer_heap, &sess->timer, &delay,
+                                      TIMER_DESTROY, sess->grp_lock);
 }
 
 
@@ -535,7 +537,7 @@ static void on_request_complete(pj_stun_session *stun_sess,
             eattr = (pj_stun_errcode_attr*)
                     pj_stun_msg_find_attr(response, PJ_STUN_ATTR_ERROR_CODE, 0);
 
-            if (eattr != NULL)
+            if (eattr != NULL && eattr->err_code)
                 err_code = eattr->err_code;
             else
                 err_code = PJ_STUN_SC_SERVER_ERROR;
@@ -932,7 +934,8 @@ static void on_sess_timer(pj_timer_heap_t *th,
 
         if (next_timer) {
             pj_time_val delay = {0, TEST_INTERVAL};
-            pj_timer_heap_schedule(th, te, &delay);
+            pj_timer_heap_schedule_w_grp_lock(th, te, &delay,
+                                              TIMER_TEST, sess->grp_lock);
         } else {
             te->id = 0;
         }
